Namespace System.Collections.Generic

   Public Interface INodeContent(Of T As INodeContent(Of T))
      Function Node(ByVal Value As clsNode(Of T)) As INodeContent(Of T)
      Function Node() As clsNode(Of T)
   End Interface

   Public Class clsNode
      Public Shared Function Create(Of T As INodeContent(Of T))(ByVal Content As T) As clsNode(Of T)
         Create = New clsNode(Of T)
         Create.Content(Content)
      End Function
   End Class

   Public Class clsNode(Of T As INodeContent(Of T))
      'Grundfunktionen hierarchischer Strukturen
      'leider finde ich keine Mglichkeit, vererbbare Baumstrukturen zu erzeugen, deren Nodes man
      '! direkt an Erfordernisse anpassen knnte.
      'Deshalb dieser Standard-Baum, dessen Nodes immerhin verschiedenerlei "Nutz-Objekte" als "Content
      '! " aufnehmen knnen.

      Private _Content As T
      Protected _Parent As clsNode(Of T)
      Protected _Key As Object = Me
      Protected ReadOnly _Children As New List(Of clsNode(Of T))
      Friend ReadOnly _ChildKeys As New List(Of Object)

      Public Overridable Function Content() As T
         Return _Content
      End Function
      Public Overridable Function Content(ByVal Value As T) As clsNode(Of T)
         _Content = Value
         _Content.Node(Me)
         Return Me
      End Function

      Public Overridable Function AddTo( _
              ByVal Parent As clsNode(Of T), _
              Optional ByVal Key As Object = Nothing) As clsNode(Of T)
         _Parent = Parent
         Parent._Children.Add(Me)
         If Key Is Nothing Then Key = Me
         _Key = Key
         Parent._ChildKeys.Add(Key)
         Return Me
      End Function

      <Obsolete()> _
      Public Function TryGetValue(ByVal Key As Object, ByRef Value As T) As Boolean
         If _ChildKeys.IndexOf(Key) < 0 Then
            Return False
         Else
            Value = Me.ChildContent(Key)
            Return True
         End If
      End Function

      Public Function GetValue(ByVal Key As Object) As T
         If _ChildKeys.IndexOf(Key) < 0 Then
            Return Nothing
         Else
            Return Me.ChildContent(Key)
         End If
      End Function

      Default Public Property ChildContent(ByVal Index As Integer) As T
         Get
            Return _Children(Index).Content
         End Get
         Set(ByVal value As T)
            _Children(Index).Content(value)
         End Set
      End Property

      Default Public Property ChildContent(ByVal Key As Object) As T
         Get
            Return _Children(_ChildKeys.IndexOf(Key)).Content
         End Get
         Set(ByVal value As T)
            _Children(_ChildKeys.IndexOf(Key)).Content(value)
         End Set
      End Property

      Public ReadOnly Property Key() As Object
         Get
            Return _Key
         End Get
      End Property

      Public ReadOnly Property AllChildren() As IEnumerable
         Get
            Static RetVal As New clsTreeEnumerable(_Children, "Children")
            Return RetVal
         End Get
      End Property

      Public ReadOnly Property AllChildren(ByVal WithRoot As Boolean) As IEnumerable
         Get
            If WithRoot = False Then Return Me.AllChildren
            'Eigentmlicher CompilerBug: RetVal mu einen anderen Namen haben als das RetVal aus der
            '! anderen AllChildren-berladung. 
            'Try This (by uncomment, then execute Regextester.Parse):
            ''Stop
            ''Static RetVal As clsTreeEnumerable
            ''If RetVal Is Nothing Then
            ''   RetVal = New clsTreeEnumerable(Me, "Children")
            ''End If
            ''Return RetVal
            Static RetVal2 As New clsTreeEnumerable(Me, "Children")
            Return RetVal2
         End Get
      End Property

      Public ReadOnly Property Children() As List(Of clsNode(Of T))
         Get
            Return _Children
         End Get
      End Property

      Public Property Parent() As clsNode(Of T)
         Get
            Return _Parent
         End Get
         Set(ByVal value As clsNode(Of T))
            Delete()
            AddTo(value, _Key)
         End Set
      End Property


      Public Sub Delete()
         With _Parent
            ._Children.Remove(Me)
            ._ChildKeys.Remove(_Key)
         End With
      End Sub

   End Class 'clsNode(Of T As INodeContent(Of T)
End Namespace