Imports System.ComponentModel
Imports System.io
Imports System.Collections.Generic
Namespace System.Windows.Forms

   <DesignerCategory("code")>
   <DefaultEvent("ItemClick")>
   Public Class MenuStripEx
      'leitet die Clicks aller ToolStripMenuItem auf den ClickHandler
      'generiert (ins Debug-Fenster) den Rumpf-Code zur Verarbeitung des ItemClick-Events
      'generiert fr RadioMenuItem Code, der deren UnterItem mit entsprechenden Enumerationen
      '! assoziiert, und mit entsprechenden Properties synchronisiert. D.h. Ein Klick auf ein
      '! UnterItem stellt einen neuen (Enumerations-)Wert der Property ein, nderungen der Property
      '! per Code werden angezeigt durchs Checken des entsprechenden MenuItems

      Inherits MenuStrip
      Implements ISupportInitialize                  'Der Designer generiert zu Beginn und am Ende von
      '!                 InitializeComponents() einen entsprechenden Aufruf von BeginInit(), EndInit()

#Region "ItemClick-Event"
      Public Class ItemClickEventArgs
         Inherits System.Windows.Forms.ToolStripItemClickedEventArgs
         Public ReadOnly FullCaption As String
         Public Shadows ReadOnly Property ClickedItem() As ToolStripMenuItem
            Get
               Return DirectCast(MyBase.ClickedItem, ToolStripMenuItem)
            End Get
         End Property
         Public Sub New(ByVal Item As ToolStripMenuItem, ByVal FullCaption As String)
            MyBase.New(Item)
            Me.FullCaption = FullCaption
         End Sub
      End Class
      Public Delegate Sub ItemClickEventHandler( _
          ByVal Sender As Object, ByVal e As ItemClickEventArgs)
      Public Event ItemClick As ItemClickEventHandler
#End Region 'ItemClick-Event

      '<zur CodeGeneriererei>
      Private _ItemClickHandlerWriter As New ObsoleteStringList
      Private _RadioItemClickHandlerWriter As New ObsoleteStringList
      Private _EnumWriter As New ObsoleteStringList
      Private _PropertyWriter As New ObsoleteStringList
      Private _HelpWriter As New ObsoleteStringList
      Const sTab As String = "    "
      Private _GenerateCode As Boolean
      '</ zur CodeGeneriererei>

      Private _FullCaptions As New Dictionary(Of ToolStripItem, String)

      Public Sub BeginInit() Implements System.ComponentModel.ISupportInitialize.BeginInit

      End Sub

      Public Sub EndInit() Implements System.ComponentModel.ISupportInitialize.EndInit
         'Initialisierung abgeschlossen; jetzt kann auf alle MenuItems zugegriffen werden
         InitMenuClicks()
         If _GenerateCode Then ExecuteGenerateCode()
      End Sub

      Private Sub InitMenuClicks()
         For Each Itm As ToolStripItem In MyBase.Items
            InitMenuClicksRecurse(Itm)
         Next
      End Sub
      Private Sub InitMenuClicksRecurse(ByVal Itm As ToolStripItem)
         If TypeOf Itm Is ToolStripMenuItem Then
            With DirectCast(Itm, ToolStripMenuItem)
               If Not .HasDropDownItems Then
                  AddHandler .Click, AddressOf ItemClickHandler
                  _FullCaptions.Add(Itm, Fullname(Itm, "OwnerItem", "Text", "\"))
               End If
               For Each itmRecurse As ToolStripItem In .DropDownItems
                  InitMenuClicksRecurse(itmRecurse)
               Next
            End With
         End If
      End Sub

      Private Sub ItemClickHandler(ByVal sender As Object, ByVal e As EventArgs)
         Dim Itm As ToolStripMenuItem = DirectCast(sender, ToolStripMenuItem)
         RaiseEvent ItemClick(Me, New ItemClickEventArgs(Itm, _FullCaptions(Itm)))
      End Sub

      <Browsable(True), _
      DefaultValue(False), _
      Category("Verhalten")> _
      Public Property GenerateCode() As Boolean
         Get
            Return _GenerateCode
         End Get
         Set(ByVal value As Boolean)
            _GenerateCode = value
         End Set
      End Property

      Private Sub ExecuteGenerateCode()
         For Each Itm As ToolStripItem In MyBase.Items
            WriteHandlerRecurse(Itm)
         Next
         If _RadioItemClickHandlerWriter.Count > 0 Then
            Debug.WriteLine(_EnumWriter.ToString(vbNewLine))
            Debug.WriteLine(_PropertyWriter.ToString(vbNewLine))
         End If
         If _ItemClickHandlerWriter.Count > 0 Then
            With _HelpWriter
               .Clear()
               .AddFormat("Private Sub {0}_ItemClick( _", Me.Name)
               .Add(sTab, sTab, "ByVal Sender As Object, _")
               .Add(sTab, sTab, "ByVal e As MenuStripEx.ItemClickEventArgs) _")
               .AddFormat(sTab & sTab & "Handles {0}.ItemClick", Me.Name)
               .Add(sTab, "select case e.FullCaption")
               .Add(_ItemClickHandlerWriter.ToString(vbNewLine))
               .Add(sTab, "End select")
               .Add("End Sub")
               Debug.WriteLine(.ToString(vbNewLine))
               Debug.WriteLine("")
            End With
         End If
         If _RadioItemClickHandlerWriter.Count > 0 Then
            With _HelpWriter
               .Clear()
               .AddFormat("Private Sub {0}_RadioItemClick( _", Me.Name)
               .Add(sTab, sTab, "ByVal Sender As Object, _")
               .Add(sTab, sTab, "ByVal e As MenuStripEx.RadioItemClickEventArgs) _")
               .AddFormat(sTab & sTab & "Handles {0}.RadioItemClick", Me.Name)
               .Add(sTab, _
                   "'Die mit dem jeweiligen RadioParent assoziierte ", _
                   "Property auf den neuen Wert einstellen")
               .Add(sTab, "'(ToDo: Nothing, all generated code)")
               .Add(sTab, "select case e.RadioParent.Text")
               .Add(_RadioItemClickHandlerWriter.ToString(vbNewLine))
               .Add(sTab, "End select")
               .Add("End Sub")
               Debug.WriteLine(.ToString(vbNewLine))
               Debug.WriteLine("")
            End With
         End If
      End Sub

      Private Sub WriteHandlerRecurse(ByVal Itm As ToolStripItem)
         If TypeOf Itm Is ToolStripMenuItem Then
            With DirectCast(Itm, ToolStripMenuItem)
               If .HasDropDownItems Then
                  For Each itmRecurse As ToolStripItem In .DropDownItems
                     WriteHandlerRecurse(itmRecurse)
                  Next
               Else
                  _ItemClickHandlerWriter.AddFormat( _
                      sTab & sTab & "case ""{0}"":", _
                     _FullCaptions(Itm))
                  _ItemClickHandlerWriter.Add( _
                      sTab, sTab, sTab, "'ToDo: Menubefehl ausfhren")
               End If
            End With
         End If
      End Sub

   End Class
End Namespace
