Imports System.Text.RegularExpressions
Imports Microsoft.VisualBasic.ControlChars
Imports System.ComponentModel

<DesignerCategory("code")>
Public Class IndentRichTextBox
   'Regexpattern knnen sehr unbersichtlich werden, da einzeilig mit beliebig verschachtelten
   '! Klammerausdrcken. Diese RTB ermglicht mehrzeilige Bearbeitung, wobei die Klammer-Struktur
   '! durch automatische Einrckungen verdeutlicht wird.

   Inherits RichTextBox

   Public ReadOnly IndentSize As Integer
   Private ReadOnly _CharWidth As Integer

   Public Sub New()
      MyBase.New()
      InitializeComponent()
      Using G As Graphics = MyBase.CreateGraphics
         'Fontsize-abhngig Einrcktiefen ermitteln. Dazu mu ein Buchstabe vermessen werden.
         Dim Format As New StringFormat
         Format.SetMeasurableCharacterRanges(New CharacterRange() {New CharacterRange(0, 1)})
         Dim Rgs As Region() = G.MeasureCharacterRanges( _
             "I", _
             MyBase.Font, _
             New RectangleF(0, 0, Short.MaxValue, Short.MaxValue), _
             Format)
         _CharWidth = CInt(Rgs(0).GetBounds(G).Width)
         IndentSize = _CharWidth * 3
      End Using
   End Sub

   Public Function LineIndent(ByVal Line As Integer) As Integer
      'Einzug der Zeile Line zurckgeben
      Dim FirstCharOfLine As Integer = 0
      For I As Integer = 0 To Line - 1
         FirstCharOfLine = MyBase.Text.IndexOf(Lf, FirstCharOfLine) + 1
         If FirstCharOfLine < 0 Then Throw New ArgumentOutOfRangeException( _
               "Line", Line, "Nicht so viele Lines vorhanden")
      Next
      MyBase.Select(FirstCharOfLine, 0)
      Return (MyBase.SelectionIndent - _CharWidth) \ IndentSize
   End Function

#Region "generated Code"
   Private Sub InitializeComponent()
      Me.SuspendLayout()
      '
      'IndentRichTextBox
      '
      Me.DetectUrls = False
      Me.Font = New System.Drawing.Font("Courier New", 9.75!, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, CType(0, Byte))
      Me.ResumeLayout(False)

   End Sub
#End Region 'generated Code


   Public Function SelectFromOneLineText(ByVal Start As Integer, ByVal Length As Integer) _
         As IndentRichTextBox
      'Um richtig zu selecten mssen die LineFeeds, die im OneLineText entfernt wurden, wieder
      '! hineingerechnet werden
      Dim S As Char() = MyBase.Text.ToCharArray
      Dim SelStart As Integer = InnerLength(S, 0, Start)
      MyBase.Select(SelStart, InnerLength(S, SelStart, Length))
      Return Me
   End Function

   Private Function InnerLength( _
         ByVal S As Char(), _
         ByVal Start As Integer, _
         ByVal Length As Integer) As Integer
      'Textposition im Text /mit/ LineFeeds, die der Position im selben einzeiligen Text entspricht
      Dim RestCount As Integer = CountChars(S, Lf, Start, Length)
      If RestCount = 0 Then Return Length
      For I As Integer = Start + Length To Integer.MaxValue
         If S(I) <> Lf Then
            RestCount -= 1
            If RestCount = 0 Then Return I + 1 - Start
         End If
      Next
   End Function

   Private Function InnerPosition( _
         ByVal S As Char(), _
         ByVal Start As Integer, _
         ByVal Pos As Integer) As Integer
      'Textposition im Text /mit/ LineFeeds, die der Position im selben einzeiligen Text entspricht
      Dim RestCount As Integer = CountChars(S, Lf, Start, Pos)
      If RestCount = 0 Then Return Pos
      For InnerPosition = Pos + 1 To MyBase.Text.Length - 1
         If S(InnerPosition) <> Lf Then
            RestCount -= 1
            If RestCount = 0 Then Exit Function
         End If
      Next
      Throw New LongMsgException( _
         "Die angegebene Position ", _
         Pos, _
         " kann im String\n'", _
         New String(S), _
         "'\nnicht gefunden werden")
   End Function

   Private Function CountChars( _
         ByVal S As Char(), _
         ByVal cMatch As Char, _
         ByVal Index As Integer, _
         ByVal Length As Integer) As Integer
      'Zhle ein Char-Array aus, weil Strings immer umkopiert werden
      Dim I As Integer = Array.IndexOf(S, cMatch, Index, Length)
      Do Until I < 0
         CountChars += 1
         I += 1
         Length -= (I - Index)
         Index = I
         I = Array.IndexOf(S, cMatch, Index, Length)
      Loop
   End Function

   Public Function OneLineText() As String
      Return MyBase.Text.Replace(Lf, "")
   End Function

   Protected Overrides Sub OnTextChanged(ByVal e As System.EventArgs)
      'AutoIndent
      Static Rgx As New Regex("((?<=(^|[^\\])(\\\\)*)[\(\)])|\n")
      Dim Indent, DeltaIndent As Integer
      Static Indents As New List(Of ValuePair(Of Integer, Integer))
      Indents.Clear()
      Dim IndentInfo As New ValuePair(Of Integer, Integer)(0, 0)    'IndentInfo enthlt Position der
      '!                                                                    Zeile und ihre Indention
      Indents.Add(IndentInfo)
      For Each Mt As Match In Rgx.Matches(MyBase.Text)
         Select Case Mt.Value
            Case "("
               DeltaIndent += 1
            Case ")"
               DeltaIndent -= 1
            Case "", Lf        'bei LineFeed oder EingabeEnde wird Indent in IndentInfo eingetragen
               If DeltaIndent > 0 Then
                  'ein berwiegen ffnender Klammern wirkt sich erst auf die folgenden Zeilen aus
                  IndentInfo.Value2 = Indent
                  Indent += DeltaIndent                               'Indent erst danach hochsetzen
               Else
                  'ein berwiegen schlieender Klammern wirkt sich auf die Zeile selbst aus.
                  Indent += DeltaIndent
                  IndentInfo.Value2 = Indent
               End If
               If Mt.Value = "" Then Exit For 'Eingabe-Ende erreicht
               'IndentInfo fr die nchste Zeile mit Zeilen-Position initialisieren
               IndentInfo = New ValuePair(Of Integer, Integer)(Mt.Index + 1, 0)
               Indents.Add(IndentInfo)
               DeltaIndent = 0
            Case Else
               Stop
         End Select
      Next
      Dim SelStart As Integer = MyBase.SelectionStart
      For Each IndentInfo In Indents
         MyBase.SelectionStart = IndentInfo.Value1
         MyBase.SelectionIndent = IndentInfo.Value2 * IndentSize + _CharWidth
      Next
      MyBase.SelectionStart = SelStart
      MyBase.OnTextChanged(e)
   End Sub

End Class